{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Queues\n",
    "\n",
    "Some programs need to retrieve data in the same order as it was stored. We call this to *enqueue* items of data. We could imagine doing this by using a list to add elements at the end and retrieve them from the front, as in the following example:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "[0, 1, 2, 3, 4, 5, 6, 7, 8, 9]\n",
      "\n",
      "Service first in queue: 0\n",
      "[1, 2, 3, 4, 5, 6, 7, 8, 9]\n",
      "\n",
      "Service first in queue: 1\n",
      "[2, 3, 4, 5, 6, 7, 8, 9]\n",
      "\n",
      "Service first in queue: 2\n",
      "[3, 4, 5, 6, 7, 8, 9]\n",
      "\n",
      "Service first in queue: 3\n",
      "[4, 5, 6, 7, 8, 9]\n",
      "\n",
      "Service first in queue: 4\n",
      "[5, 6, 7, 8, 9]\n",
      "\n",
      "Service first in queue: 5\n",
      "[6, 7, 8, 9]\n",
      "\n",
      "Service first in queue: 6\n",
      "[7, 8, 9]\n",
      "\n",
      "Service first in queue: 7\n",
      "[8, 9]\n",
      "\n",
      "Service first in queue: 8\n",
      "[9]\n",
      "\n",
      "Service first in queue: 9\n",
      "[]\n",
      "\n"
     ]
    }
   ],
   "source": [
    "q = []\n",
    "\n",
    "for i in range(10):\n",
    "    # enqueue by placing at the end:\n",
    "    q.append(i)\n",
    "print(q)\n",
    "print()\n",
    "\n",
    "for i in range(10):\n",
    "    # dequeue by removing from the front:\n",
    "    print('Service first in queue:', q.pop(0))\n",
    "    print(q)\n",
    "    print()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "The problem is that we know that ```pop(0)``` is linear time! We would like to have constant time instead (not more expensive than updating / reading an array). \n",
    "\n",
    "This can be achieved by keeping track of the first and the last element and keeping the elements together via links. We call the pair of an element and a link a Node. We use a class with the name ```_Node``` for these nodes (remember that we use the convention of using an underscore at the begining of a name to indicate that it is an auxiliary function or name). As you can see in the definintion in next cell, a node has two instance variables: an element (item) and a link (next).\n",
    "\n",
    "The class for queues is called ```Queue``` (no underscore!) and has two instance variables: the first node and the last node! It has methods to add items (```enqueue```)and to remove items (```dequeue```). Also for testing emptiness, for the number of elements and to make a string of the content.\n",
    "\n",
    "You to inspect the code! Confirm that all operations are constant time (except ```__str__```!)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [],
   "source": [
    "# From the book Introduction to Programming in Python, chapter 4.3 on Stacks and Queues:\n",
    "# https://introcs.cs.princeton.edu/python/43stack/\n",
    "\n",
    "#----------------------------------------------------------------------\n",
    "\n",
    "# A _Node object references an item and a next _Node object.\n",
    "# A Queue object is composed of _Node objects.\n",
    "\n",
    "class _Node:\n",
    "    def __init__(self, item, next):\n",
    "        self.item = item  # Reference to an item\n",
    "        self.next = next  # Reference to the next _Node object\n",
    "\n",
    "\n",
    "\n",
    "class Queue:\n",
    "\n",
    "    #-------------------------------------------------------------------\n",
    "\n",
    "    # Construct the Queue object self as an empty Queue object.\n",
    "\n",
    "    def __init__(self):\n",
    "        self._first  = None  # Reference to first _Node\n",
    "        self._last   = None   # Reference to last _Node\n",
    "        self._length = 0    # Number of items\n",
    "\n",
    "    #-------------------------------------------------------------------\n",
    "\n",
    "    # Return True if self is empty, and False otherwise.\n",
    "\n",
    "    def is_empty(self):\n",
    "        return self._first is None\n",
    "\n",
    "    #-------------------------------------------------------------------\n",
    "\n",
    "    # Add item to the end of self.\n",
    "\n",
    "    def enqueue(self, item):\n",
    "        oldLast = self._last\n",
    "        self._last = _Node(item, None)\n",
    "        if self.is_empty():\n",
    "            self._first = self._last\n",
    "        else:\n",
    "            oldLast.next = self._last\n",
    "        self._length += 1\n",
    "\n",
    "    #-------------------------------------------------------------------\n",
    "\n",
    "    # Remove the first item of self and return it.\n",
    "\n",
    "    def dequeue(self):\n",
    "        item = self._first.item\n",
    "        self._first = self._first.next\n",
    "        if self.is_empty():\n",
    "            self._last = None\n",
    "        self._length -= 1\n",
    "        return item\n",
    "\n",
    "    #-------------------------------------------------------------------\n",
    "\n",
    "    # Return the number of items in self.\n",
    "\n",
    "    def __len__(self):\n",
    "        return self._length\n",
    "\n",
    "    #-------------------------------------------------------------------\n",
    "\n",
    "    # Return a string representation of self.\n",
    "\n",
    "    def __str__(self):\n",
    "        s = ''\n",
    "        cur = self._first\n",
    "        while cur is not None:\n",
    "            s += str(cur.item) + ' '\n",
    "            cur = cur.next\n",
    "        return s\n",
    "\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Here is the same example as before but using this implementation for queues:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 35,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "0 1 2 3 4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 0\n",
      "1 2 3 4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 1\n",
      "2 3 4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 2\n",
      "3 4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 3\n",
      "4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 4\n",
      "5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 5\n",
      "6 7 8 9 \n",
      "\n",
      "Service first in queue: 6\n",
      "7 8 9 \n",
      "\n",
      "Service first in queue: 7\n",
      "8 9 \n",
      "\n",
      "Service first in queue: 8\n",
      "9 \n",
      "\n",
      "Service first in queue: 9\n",
      "\n",
      "\n"
     ]
    }
   ],
   "source": [
    "q = Queue()\n",
    "\n",
    "for i in range(10):\n",
    "    q.enqueue(i)\n",
    "    \n",
    "print(q)\n",
    "print()\n",
    "\n",
    "for i in range(10):\n",
    "    # dequeue by removing from the front:\n",
    "    print('Service first in queue:', q.dequeue())\n",
    "    print(q)\n",
    "    print()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 36,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "0 1 2 3 4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 0\n",
      "1 2 3 4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 1\n",
      "2 3 4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 2\n",
      "3 4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 3\n",
      "4 5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 4\n",
      "5 6 7 8 9 \n",
      "\n",
      "Service first in queue: 5\n",
      "6 7 8 9 \n",
      "\n",
      "Service first in queue: 6\n",
      "7 8 9 \n",
      "\n",
      "Service first in queue: 7\n",
      "8 9 \n",
      "\n",
      "Service first in queue: 8\n",
      "9 \n",
      "\n",
      "Service first in queue: 9\n",
      "\n",
      "\n"
     ]
    }
   ],
   "source": [
    "# In case we do not know how many times we can dequeue we can test for emptyness:\n",
    "\n",
    "q = Queue()\n",
    "\n",
    "for i in range(10):\n",
    "    q.enqueue(i)\n",
    "    \n",
    "print(q)\n",
    "print()\n",
    "\n",
    "while not q.is_empty():\n",
    "    # dequeue by removing from the front:\n",
    "    print('Service first in queue:', q.dequeue())\n",
    "    print(q)\n",
    "    print()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Listing the contents of a folder\n",
    "\n",
    "And now an example. Say you want to list the contents of a folder and of all folders under this one. In Python you can get a list of names of files and folders (directories) using the method ```listdir``` in the module ```os```.\n",
    "\n",
    "Here is how my directory for this lecture notes looks like:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 37,
   "metadata": {},
   "outputs": [],
   "source": [
    "import os"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 38,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "design\n",
      "ds\n",
      "intro.md\n",
      "panda.png\n",
      "algs\n",
      "_toc.yml\n",
      "_build\n",
      "_config.yml\n",
      ".ipynb_checkpoints\n",
      "overview\n",
      "panda.gif\n"
     ]
    }
   ],
   "source": [
    "for name in os.listdir('..'):\n",
    "    print(name)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Not all of these are directories, so if we want to list the contents of the directories we need to test whether a name corresponds to a directory. This can be done in Python using the method ```is_dir``` in the module ```os.path```. \n",
    "\n",
    "So, here we list all sub-directories also:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 46,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "design\n",
      "['dc-mergesort.ipynb', 'dc-inversions.ipynb', 'dp-knapsack.ipynb', 'dp-wis.ipynb', 'dp-fib.ipynb', 'intro.md', 'dc-intro.md', '.ipynb_checkpoints', 'dp-intro.md']\n",
      "ds\n",
      "['intro.md', 'lists.ipynb', 'lecture5.md', 'queues.ipynb', 'classes.ipynb', '.ipynb_checkpoints']\n",
      "intro.md\n",
      "panda.png\n",
      "algs\n",
      "['programming-exercises-1.ipynb', 'execution-time.md', 'theory-exercises-1.md', 'math-and-experiments.ipynb', 'et.ipynb', 'intro.md', 'mul.ipynb', 'loops.ipynb', 'recursion.ipynb', '.ipynb_checkpoints', 'algs-lecture.md', 'primes.ipynb']\n",
      "_toc.yml\n",
      "_build\n",
      "['.DS_Store', 'html', 'jupyter_execute', '.doctrees']\n",
      "_config.yml\n",
      ".ipynb_checkpoints\n",
      "['week2-divide-and-conquer-and-analysis-checkpoint.ipynb']\n",
      "overview\n",
      "['overview.md', 'sw-dev-cycle-ch7-sedgewick-cs.png', '.ipynb_checkpoints']\n",
      "panda.gif\n"
     ]
    }
   ],
   "source": [
    "import os\n",
    "for name in os.listdir('..'):\n",
    "    print(name)\n",
    "    if os.path.isdir('../' + name):\n",
    "        print(os.listdir('../' + name))\n",
    "        "
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "We would like to list as long as there are files, as deep as needed. We do it level by level, using a queue! This algorithm is called Breadth First Search. \n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 44,
   "metadata": {},
   "outputs": [],
   "source": [
    "def list_dir_tree(path):\n",
    "    q = Queue()\n",
    "    q.enqueue(path)\n",
    "    \n",
    "    while not q.is_empty():\n",
    "        path = q.dequeue()\n",
    "        print(path)\n",
    "        if os.path.isdir(path):\n",
    "            for name in os.listdir(path):\n",
    "                q.enqueue(path + '/' + name)\n",
    "     \n",
    "    "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 49,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "..\n",
      "../design\n",
      "../ds\n",
      "../intro.md\n",
      "../panda.png\n",
      "../algs\n",
      "../_toc.yml\n",
      "../_build\n",
      "../_config.yml\n",
      "../.ipynb_checkpoints\n",
      "../overview\n",
      "../panda.gif\n",
      "../design/dc-mergesort.ipynb\n",
      "../design/dc-inversions.ipynb\n",
      "../design/dp-knapsack.ipynb\n",
      "../design/dp-wis.ipynb\n",
      "../design/dp-fib.ipynb\n",
      "../design/intro.md\n",
      "../design/dc-intro.md\n",
      "../design/.ipynb_checkpoints\n",
      "../design/dp-intro.md\n",
      "../ds/intro.md\n",
      "../ds/lists.ipynb\n",
      "../ds/lecture5.md\n",
      "../ds/queues.ipynb\n",
      "../ds/classes.ipynb\n",
      "../ds/.ipynb_checkpoints\n",
      "../algs/programming-exercises-1.ipynb\n",
      "../algs/execution-time.md\n",
      "../algs/theory-exercises-1.md\n",
      "../algs/math-and-experiments.ipynb\n",
      "../algs/et.ipynb\n",
      "../algs/intro.md\n",
      "../algs/mul.ipynb\n",
      "../algs/loops.ipynb\n",
      "../algs/recursion.ipynb\n",
      "../algs/.ipynb_checkpoints\n",
      "../algs/algs-lecture.md\n",
      "../algs/primes.ipynb\n",
      "../_build/.DS_Store\n",
      "../_build/html\n",
      "../_build/jupyter_execute\n",
      "../_build/.doctrees\n",
      "../.ipynb_checkpoints/week2-divide-and-conquer-and-analysis-checkpoint.ipynb\n",
      "../overview/overview.md\n",
      "../overview/sw-dev-cycle-ch7-sedgewick-cs.png\n",
      "../overview/.ipynb_checkpoints\n",
      "../design/.ipynb_checkpoints/dc-mergesort-checkpoint.ipynb\n",
      "../design/.ipynb_checkpoints/dp-knapsack-checkpoint.ipynb\n",
      "../design/.ipynb_checkpoints/dp-wis-checkpoint.ipynb\n",
      "../design/.ipynb_checkpoints/dp-fib-checkpoint.ipynb\n",
      "../design/.ipynb_checkpoints/dc-inversions-checkpoint.ipynb\n",
      "../ds/.ipynb_checkpoints/lists-checkpoint.ipynb\n",
      "../ds/.ipynb_checkpoints/queues-checkpoint.ipynb\n",
      "../ds/.ipynb_checkpoints/Untitled-checkpoint.ipynb\n",
      "../ds/.ipynb_checkpoints/classes-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/recursion-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/math-and-experiments-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/loops-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/et-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/mul-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/programming-exercises-1-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/primes-checkpoint.ipynb\n",
      "../_build/html/design\n",
      "../_build/html/week2-divide-and-conquer-and-analysis.html\n",
      "../_build/html/index.html\n",
      "../_build/html/.DS_Store\n",
      "../_build/html/ds\n",
      "../_build/html/searchindex.js\n",
      "../_build/html/_panels_static\n",
      "../_build/html/_sources\n",
      "../_build/html/algs\n",
      "../_build/html/_static\n",
      "../_build/html/intro.html\n",
      "../_build/html/.buildinfo\n",
      "../_build/html/_images\n",
      "../_build/html/genindex.html\n",
      "../_build/html/search.html\n",
      "../_build/html/overview\n",
      "../_build/html/objects.inv\n",
      "../_build/html/reports\n",
      "../_build/jupyter_execute/design\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis_10_0.png\n",
      "../_build/jupyter_execute/ds\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis_11_0.png\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis.ipynb\n",
      "../_build/jupyter_execute/algs\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis.py\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis_9_0.png\n",
      "../_build/.doctrees/design\n",
      "../_build/.doctrees/week2-divide-and-conquer-and-analysis.doctree\n",
      "../_build/.doctrees/ds\n",
      "../_build/.doctrees/algs\n",
      "../_build/.doctrees/intro.doctree\n",
      "../_build/.doctrees/environment.pickle\n",
      "../_build/.doctrees/overview\n",
      "../_build/.doctrees/glue_cache.json\n",
      "../_build/html/design/dp-knapsack.html\n",
      "../_build/html/design/dc-inversions.html\n",
      "../_build/html/design/dp-intro.html\n",
      "../_build/html/design/dc-intro.html\n",
      "../_build/html/design/intro.html\n",
      "../_build/html/design/dp-wis.html\n",
      "../_build/html/design/dp-fib.html\n",
      "../_build/html/design/dc-mergesort.html\n",
      "../_build/html/ds/queues.html\n",
      "../_build/html/ds/lists.html\n",
      "../_build/html/ds/intro.html\n",
      "../_build/html/ds/lecture5.html\n",
      "../_build/html/ds/classes.html\n",
      "../_build/html/_panels_static/panels-variables.06eb56fa6e07937060861dad626602ad.css\n",
      "../_build/html/_panels_static/panels-main.c949a650a448cc0ae9fd3441c0e17fb0.css\n",
      "../_build/html/_sources/design\n",
      "../_build/html/_sources/.DS_Store\n",
      "../_build/html/_sources/ds\n",
      "../_build/html/_sources/intro.md\n",
      "../_build/html/_sources/week2-divide-and-conquer-and-analysis.ipynb\n",
      "../_build/html/_sources/algs\n",
      "../_build/html/_sources/overview\n",
      "../_build/html/algs/mul.html\n",
      "../_build/html/algs/programming-exercises-1.html\n",
      "../_build/html/algs/theory-exercises-1.html\n",
      "../_build/html/algs/multiplication.html\n",
      "../_build/html/algs/et.html\n",
      "../_build/html/algs/intro.html\n",
      "../_build/html/algs/primes.html\n",
      "../_build/html/algs/algs-lecture.html\n",
      "../_build/html/algs/loops.html\n",
      "../_build/html/algs/recursion.html\n",
      "../_build/html/algs/math-and-experiments.html\n",
      "../_build/html/algs/execution-time.html\n",
      "../_build/html/algs/about-primes.html\n",
      "../_build/html/algs/for-primes-time.html\n",
      "../_build/html/_static/plus.png\n",
      "../_build/html/_static/underscore.js\n",
      "../_build/html/_static/webpack-macros.html\n",
      "../_build/html/_static/.DS_Store\n",
      "../_build/html/_static/documentation_options.js\n",
      "../_build/html/_static/searchtools.js\n",
      "../_build/html/_static/css\n",
      "../_build/html/_static/check-solid.svg\n",
      "../_build/html/_static/images\n",
      "../_build/html/_static/js\n",
      "../_build/html/_static/clipboard.min.js\n",
      "../_build/html/_static/file.png\n",
      "../_build/html/_static/copybutton.js\n",
      "../_build/html/_static/panda.png\n",
      "../_build/html/_static/jquery-3.5.1.js\n",
      "../_build/html/_static/__init__.py\n",
      "../_build/html/_static/__pycache__\n",
      "../_build/html/_static/language_data.js\n",
      "../_build/html/_static/copy-button.svg\n",
      "../_build/html/_static/copybutton_funcs.js\n",
      "../_build/html/_static/minus.png\n",
      "../_build/html/_static/mystnb.css\n",
      "../_build/html/_static/panels-variables.06eb56fa6e07937060861dad626602ad.css\n",
      "../_build/html/_static/basic.css\n",
      "../_build/html/_static/sphinx-book-theme.css\n",
      "../_build/html/_static/sphinx-book-theme.e2363ea40746bee74734a24ffefccd78.css\n",
      "../_build/html/_static/panels-main.c949a650a448cc0ae9fd3441c0e17fb0.css\n",
      "../_build/html/_static/pygments.css\n",
      "../_build/html/_static/sphinx-thebe.js\n",
      "../_build/html/_static/copybutton.css\n",
      "../_build/html/_static/doctools.js\n",
      "../_build/html/_static/sphinx-thebe.css\n",
      "../_build/html/_static/togglebutton.js\n",
      "../_build/html/_static/vendor\n",
      "../_build/html/_static/sphinx-book-theme.d59cb220de22ca1c485ebbdc042f0030.js\n",
      "../_build/html/_static/jquery.js\n",
      "../_build/html/_static/togglebutton.css\n",
      "../_build/html/_static/underscore-1.13.1.js\n",
      "../_build/html/_images/math-and-experiments_19_0.png\n",
      "../_build/html/_images/week2-divide-and-conquer-and-analysis_10_0.png\n",
      "../_build/html/_images/week2-divide-and-conquer-and-analysis_11_0.png\n",
      "../_build/html/_images/math-and-experiments_25_0.png\n",
      "../_build/html/_images/math-and-experiments_21_0.png\n",
      "../_build/html/_images/math-and-experiments_23_0.png\n",
      "../_build/html/_images/math-and-experiments_12_0.png\n",
      "../_build/html/_images/dc-mergesort_14_0.png\n",
      "../_build/html/_images/math-and-experiments_14_0.png\n",
      "../_build/html/_images/dc-mergesort_12_0.png\n",
      "../_build/html/_images/math-and-experiments_16_0.png\n",
      "../_build/html/_images/math-and-experiments_35_0.png\n",
      "../_build/html/_images/lists_4_0.png\n",
      "../_build/html/_images/week2-divide-and-conquer-and-analysis_9_0.png\n",
      "../_build/html/_images/dc-inversions_11_0.png\n",
      "../_build/html/overview/overview.html\n",
      "../_build/html/reports/dp-fib.log\n",
      "../_build/html/reports/mul.log\n",
      "../_build/html/reports/week2-divide-and-conquer-and-analysis.log\n",
      "../_build/jupyter_execute/design/dc-mergesort.ipynb\n",
      "../_build/jupyter_execute/design/dp-wis.py\n",
      "../_build/jupyter_execute/design/dc-inversions.ipynb\n",
      "../_build/jupyter_execute/design/dp-fib.py\n",
      "../_build/jupyter_execute/design/dp-knapsack.py\n",
      "../_build/jupyter_execute/design/dp-knapsack.ipynb\n",
      "../_build/jupyter_execute/design/dp-wis.ipynb\n",
      "../_build/jupyter_execute/design/dp-fib.ipynb\n",
      "../_build/jupyter_execute/design/dc-mergesort_14_0.png\n",
      "../_build/jupyter_execute/design/dc-mergesort_12_0.png\n",
      "../_build/jupyter_execute/design/dc-inversions_11_0.png\n",
      "../_build/jupyter_execute/design/dc-mergesort.py\n",
      "../_build/jupyter_execute/design/dc-inversions.py\n",
      "../_build/jupyter_execute/ds/queues.py\n",
      "../_build/jupyter_execute/ds/lists.py\n",
      "../_build/jupyter_execute/ds/lists.ipynb\n",
      "../_build/jupyter_execute/ds/queues.ipynb\n",
      "../_build/jupyter_execute/ds/classes.ipynb\n",
      "../_build/jupyter_execute/ds/lists_4_0.png\n",
      "../_build/jupyter_execute/ds/classes.py\n",
      "../_build/jupyter_execute/algs/primes.py\n",
      "../_build/jupyter_execute/algs/programming-exercises-1.ipynb\n",
      "../_build/jupyter_execute/algs/math-and-experiments_19_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments.ipynb\n",
      "../_build/jupyter_execute/algs/et.py\n",
      "../_build/jupyter_execute/algs/et.ipynb\n",
      "../_build/jupyter_execute/algs/math-and-experiments_25_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments_21_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments.py\n",
      "../_build/jupyter_execute/algs/mul.ipynb\n",
      "../_build/jupyter_execute/algs/loops.ipynb\n",
      "../_build/jupyter_execute/algs/math-and-experiments_23_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments_12_0.png\n",
      "../_build/jupyter_execute/algs/loops.py\n",
      "../_build/jupyter_execute/algs/mul.py\n",
      "../_build/jupyter_execute/algs/recursion.ipynb\n",
      "../_build/jupyter_execute/algs/math-and-experiments_14_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments_16_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments_35_0.png\n",
      "../_build/jupyter_execute/algs/programming-exercises-1.py\n",
      "../_build/jupyter_execute/algs/recursion.py\n",
      "../_build/jupyter_execute/algs/primes.ipynb\n",
      "../_build/.doctrees/design/dc-intro.doctree\n",
      "../_build/.doctrees/design/dp-fib.doctree\n",
      "../_build/.doctrees/design/dp-wis.doctree\n",
      "../_build/.doctrees/design/dc-inversions.doctree\n",
      "../_build/.doctrees/design/intro.doctree\n",
      "../_build/.doctrees/design/dc-mergesort.doctree\n",
      "../_build/.doctrees/design/dp-knapsack.doctree\n",
      "../_build/.doctrees/design/dp-intro.doctree\n",
      "../_build/.doctrees/ds/classes.doctree\n",
      "../_build/.doctrees/ds/lists.doctree\n",
      "../_build/.doctrees/ds/lecture5.doctree\n",
      "../_build/.doctrees/ds/intro.doctree\n",
      "../_build/.doctrees/ds/queues.doctree\n",
      "../_build/.doctrees/algs/loops.doctree\n",
      "../_build/.doctrees/algs/programming-exercises-1.doctree\n",
      "../_build/.doctrees/algs/for-primes-time.doctree\n",
      "../_build/.doctrees/algs/algs-lecture.doctree\n",
      "../_build/.doctrees/algs/theory-exercises-1.doctree\n",
      "../_build/.doctrees/algs/mul.doctree\n",
      "../_build/.doctrees/algs/recursion.doctree\n",
      "../_build/.doctrees/algs/multiplication.doctree\n",
      "../_build/.doctrees/algs/primes.doctree\n",
      "../_build/.doctrees/algs/execution-time.doctree\n",
      "../_build/.doctrees/algs/intro.doctree\n",
      "../_build/.doctrees/algs/math-and-experiments.doctree\n",
      "../_build/.doctrees/algs/about-primes.doctree\n",
      "../_build/.doctrees/algs/et.doctree\n",
      "../_build/.doctrees/overview/overview.doctree\n",
      "../_build/html/_sources/design/dc-mergesort.ipynb\n",
      "../_build/html/_sources/design/dc-inversions.ipynb\n",
      "../_build/html/_sources/design/dp-knapsack.ipynb\n",
      "../_build/html/_sources/design/dp-wis.ipynb\n",
      "../_build/html/_sources/design/dp-fib.ipynb\n",
      "../_build/html/_sources/design/intro.md\n",
      "../_build/html/_sources/design/dc-intro.md\n",
      "../_build/html/_sources/design/dp-intro.md\n",
      "../_build/html/_sources/ds/intro.md\n",
      "../_build/html/_sources/ds/lists.ipynb\n",
      "../_build/html/_sources/ds/lecture5.md\n",
      "../_build/html/_sources/ds/queues.ipynb\n",
      "../_build/html/_sources/ds/classes.ipynb\n",
      "../_build/html/_sources/algs/multiplication.md\n",
      "../_build/html/_sources/algs/programming-exercises-1.ipynb\n",
      "../_build/html/_sources/algs/execution-time.md\n",
      "../_build/html/_sources/algs/theory-exercises-1.md\n",
      "../_build/html/_sources/algs/math-and-experiments.ipynb\n",
      "../_build/html/_sources/algs/et.ipynb\n",
      "../_build/html/_sources/algs/intro.md\n",
      "../_build/html/_sources/algs/mul.ipynb\n",
      "../_build/html/_sources/algs/about-primes.md\n",
      "../_build/html/_sources/algs/loops.ipynb\n",
      "../_build/html/_sources/algs/recursion.ipynb\n",
      "../_build/html/_sources/algs/primes.md\n",
      "../_build/html/_sources/algs/algs-lecture.md\n",
      "../_build/html/_sources/algs/primes.ipynb\n",
      "../_build/html/_sources/algs/for-primes-time.md\n",
      "../_build/html/_sources/overview/overview.md\n",
      "../_build/html/_static/css/blank.css\n",
      "../_build/html/_static/css/theme.css\n",
      "../_build/html/_static/css/index.ff1ffe594081f20da1ef19478df9384b.css\n",
      "../_build/html/_static/images/logo_colab.png\n",
      "../_build/html/_static/images/logo_binder.svg\n",
      "../_build/html/_static/images/logo_jupyterhub.svg\n",
      "../_build/html/_static/js/index.be7d3bbb2ef33a8344ce.js\n",
      "../_build/html/_static/__pycache__/__init__.cpython-310.pyc\n",
      "../_build/html/_static/vendor/.DS_Store\n",
      "../_build/html/_static/vendor/fontawesome\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0\n",
      "../_build/html/_static/vendor/fontawesome/.DS_Store\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/css\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/LICENSE.txt\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/css/all.min.css\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.ttf\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.svg\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.woff2\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.eot\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.svg\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.woff\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.eot\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.svg\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.woff\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.ttf\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.woff2\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.woff2\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.woff\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.ttf\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.eot\n"
     ]
    }
   ],
   "source": [
    "list_dir_tree('..')"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Make sure you understand the algorithm and the output!  Can you find the source of this notebook? It is \n",
    "\n",
    "```../ds/queues.ipynb```.\n",
    "\n",
    "\n",
    "The first thing is the starting path, then file and directory names directly under it, then next level for those names that were directories, etc. \n",
    "\n",
    "I would recommend you to try to follow the algorithm by hand with a directory structure such as\n",
    "```text\n",
    "A\n",
    "    B\n",
    "        e\n",
    "        f\n",
    "        \n",
    "    c\n",
    "    \n",
    "    D\n",
    "        g\n",
    "        H\n",
    "            i \n",
    "            j\n",
    "```\n",
    "where capital letters are dictionaries and small letters are files. Make sure you keep track of the queue! You should produce the output:\n",
    "\n",
    "```text\n",
    "A\n",
    "\n",
    "A/B\n",
    "A/c\n",
    "A/D\n",
    "\n",
    "A/B/e\n",
    "A/B/f\n",
    "A/D/g\n",
    "A/D/H\n",
    "\n",
    "A/D/H/i\n",
    "A/D/H/j\n",
    "```\n",
    "where I have added new lines to mark the depth levels."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "By the way, what would the algorithm do if we used a stack instead?"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 52,
   "metadata": {},
   "outputs": [],
   "source": [
    "def list_dir_tree_stack(path):\n",
    "    s = []\n",
    "    s.append(path)\n",
    "    \n",
    "    while not s == []:\n",
    "        path = s.pop()\n",
    "        print(path)\n",
    "        if os.path.isdir(path):\n",
    "            for name in os.listdir(path):\n",
    "                s.append(path + '/' + name)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 53,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "..\n",
      "../panda.gif\n",
      "../overview\n",
      "../overview/.ipynb_checkpoints\n",
      "../overview/sw-dev-cycle-ch7-sedgewick-cs.png\n",
      "../overview/overview.md\n",
      "../.ipynb_checkpoints\n",
      "../.ipynb_checkpoints/week2-divide-and-conquer-and-analysis-checkpoint.ipynb\n",
      "../_config.yml\n",
      "../_build\n",
      "../_build/.doctrees\n",
      "../_build/.doctrees/glue_cache.json\n",
      "../_build/.doctrees/overview\n",
      "../_build/.doctrees/overview/overview.doctree\n",
      "../_build/.doctrees/environment.pickle\n",
      "../_build/.doctrees/intro.doctree\n",
      "../_build/.doctrees/algs\n",
      "../_build/.doctrees/algs/et.doctree\n",
      "../_build/.doctrees/algs/about-primes.doctree\n",
      "../_build/.doctrees/algs/math-and-experiments.doctree\n",
      "../_build/.doctrees/algs/intro.doctree\n",
      "../_build/.doctrees/algs/execution-time.doctree\n",
      "../_build/.doctrees/algs/primes.doctree\n",
      "../_build/.doctrees/algs/multiplication.doctree\n",
      "../_build/.doctrees/algs/recursion.doctree\n",
      "../_build/.doctrees/algs/mul.doctree\n",
      "../_build/.doctrees/algs/theory-exercises-1.doctree\n",
      "../_build/.doctrees/algs/algs-lecture.doctree\n",
      "../_build/.doctrees/algs/for-primes-time.doctree\n",
      "../_build/.doctrees/algs/programming-exercises-1.doctree\n",
      "../_build/.doctrees/algs/loops.doctree\n",
      "../_build/.doctrees/ds\n",
      "../_build/.doctrees/ds/queues.doctree\n",
      "../_build/.doctrees/ds/intro.doctree\n",
      "../_build/.doctrees/ds/lecture5.doctree\n",
      "../_build/.doctrees/ds/lists.doctree\n",
      "../_build/.doctrees/ds/classes.doctree\n",
      "../_build/.doctrees/week2-divide-and-conquer-and-analysis.doctree\n",
      "../_build/.doctrees/design\n",
      "../_build/.doctrees/design/dp-intro.doctree\n",
      "../_build/.doctrees/design/dp-knapsack.doctree\n",
      "../_build/.doctrees/design/dc-mergesort.doctree\n",
      "../_build/.doctrees/design/intro.doctree\n",
      "../_build/.doctrees/design/dc-inversions.doctree\n",
      "../_build/.doctrees/design/dp-wis.doctree\n",
      "../_build/.doctrees/design/dp-fib.doctree\n",
      "../_build/.doctrees/design/dc-intro.doctree\n",
      "../_build/jupyter_execute\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis_9_0.png\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis.py\n",
      "../_build/jupyter_execute/algs\n",
      "../_build/jupyter_execute/algs/primes.ipynb\n",
      "../_build/jupyter_execute/algs/recursion.py\n",
      "../_build/jupyter_execute/algs/programming-exercises-1.py\n",
      "../_build/jupyter_execute/algs/math-and-experiments_35_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments_16_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments_14_0.png\n",
      "../_build/jupyter_execute/algs/recursion.ipynb\n",
      "../_build/jupyter_execute/algs/mul.py\n",
      "../_build/jupyter_execute/algs/loops.py\n",
      "../_build/jupyter_execute/algs/math-and-experiments_12_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments_23_0.png\n",
      "../_build/jupyter_execute/algs/loops.ipynb\n",
      "../_build/jupyter_execute/algs/mul.ipynb\n",
      "../_build/jupyter_execute/algs/math-and-experiments.py\n",
      "../_build/jupyter_execute/algs/math-and-experiments_21_0.png\n",
      "../_build/jupyter_execute/algs/math-and-experiments_25_0.png\n",
      "../_build/jupyter_execute/algs/et.ipynb\n",
      "../_build/jupyter_execute/algs/et.py\n",
      "../_build/jupyter_execute/algs/math-and-experiments.ipynb\n",
      "../_build/jupyter_execute/algs/math-and-experiments_19_0.png\n",
      "../_build/jupyter_execute/algs/programming-exercises-1.ipynb\n",
      "../_build/jupyter_execute/algs/primes.py\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis.ipynb\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis_11_0.png\n",
      "../_build/jupyter_execute/ds\n",
      "../_build/jupyter_execute/ds/classes.py\n",
      "../_build/jupyter_execute/ds/lists_4_0.png\n",
      "../_build/jupyter_execute/ds/classes.ipynb\n",
      "../_build/jupyter_execute/ds/queues.ipynb\n",
      "../_build/jupyter_execute/ds/lists.ipynb\n",
      "../_build/jupyter_execute/ds/lists.py\n",
      "../_build/jupyter_execute/ds/queues.py\n",
      "../_build/jupyter_execute/week2-divide-and-conquer-and-analysis_10_0.png\n",
      "../_build/jupyter_execute/design\n",
      "../_build/jupyter_execute/design/dc-inversions.py\n",
      "../_build/jupyter_execute/design/dc-mergesort.py\n",
      "../_build/jupyter_execute/design/dc-inversions_11_0.png\n",
      "../_build/jupyter_execute/design/dc-mergesort_12_0.png\n",
      "../_build/jupyter_execute/design/dc-mergesort_14_0.png\n",
      "../_build/jupyter_execute/design/dp-fib.ipynb\n",
      "../_build/jupyter_execute/design/dp-wis.ipynb\n",
      "../_build/jupyter_execute/design/dp-knapsack.ipynb\n",
      "../_build/jupyter_execute/design/dp-knapsack.py\n",
      "../_build/jupyter_execute/design/dp-fib.py\n",
      "../_build/jupyter_execute/design/dc-inversions.ipynb\n",
      "../_build/jupyter_execute/design/dp-wis.py\n",
      "../_build/jupyter_execute/design/dc-mergesort.ipynb\n",
      "../_build/html\n",
      "../_build/html/reports\n",
      "../_build/html/reports/week2-divide-and-conquer-and-analysis.log\n",
      "../_build/html/reports/mul.log\n",
      "../_build/html/reports/dp-fib.log\n",
      "../_build/html/objects.inv\n",
      "../_build/html/overview\n",
      "../_build/html/overview/overview.html\n",
      "../_build/html/search.html\n",
      "../_build/html/genindex.html\n",
      "../_build/html/_images\n",
      "../_build/html/_images/dc-inversions_11_0.png\n",
      "../_build/html/_images/week2-divide-and-conquer-and-analysis_9_0.png\n",
      "../_build/html/_images/lists_4_0.png\n",
      "../_build/html/_images/math-and-experiments_35_0.png\n",
      "../_build/html/_images/math-and-experiments_16_0.png\n",
      "../_build/html/_images/dc-mergesort_12_0.png\n",
      "../_build/html/_images/math-and-experiments_14_0.png\n",
      "../_build/html/_images/dc-mergesort_14_0.png\n",
      "../_build/html/_images/math-and-experiments_12_0.png\n",
      "../_build/html/_images/math-and-experiments_23_0.png\n",
      "../_build/html/_images/math-and-experiments_21_0.png\n",
      "../_build/html/_images/math-and-experiments_25_0.png\n",
      "../_build/html/_images/week2-divide-and-conquer-and-analysis_11_0.png\n",
      "../_build/html/_images/week2-divide-and-conquer-and-analysis_10_0.png\n",
      "../_build/html/_images/math-and-experiments_19_0.png\n",
      "../_build/html/.buildinfo\n",
      "../_build/html/intro.html\n",
      "../_build/html/_static\n",
      "../_build/html/_static/underscore-1.13.1.js\n",
      "../_build/html/_static/togglebutton.css\n",
      "../_build/html/_static/jquery.js\n",
      "../_build/html/_static/sphinx-book-theme.d59cb220de22ca1c485ebbdc042f0030.js\n",
      "../_build/html/_static/vendor\n",
      "../_build/html/_static/vendor/fontawesome\n",
      "../_build/html/_static/vendor/fontawesome/.DS_Store\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/LICENSE.txt\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.eot\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.ttf\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.woff\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.woff2\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.woff2\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.ttf\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.woff\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.svg\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.eot\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.woff\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-brands-400.svg\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.eot\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.woff2\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-regular-400.svg\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/webfonts/fa-solid-900.ttf\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/css\n",
      "../_build/html/_static/vendor/fontawesome/5.13.0/css/all.min.css\n",
      "../_build/html/_static/vendor/.DS_Store\n",
      "../_build/html/_static/togglebutton.js\n",
      "../_build/html/_static/sphinx-thebe.css\n",
      "../_build/html/_static/doctools.js\n",
      "../_build/html/_static/copybutton.css\n",
      "../_build/html/_static/sphinx-thebe.js\n",
      "../_build/html/_static/pygments.css\n",
      "../_build/html/_static/panels-main.c949a650a448cc0ae9fd3441c0e17fb0.css\n",
      "../_build/html/_static/sphinx-book-theme.e2363ea40746bee74734a24ffefccd78.css\n",
      "../_build/html/_static/sphinx-book-theme.css\n",
      "../_build/html/_static/basic.css\n",
      "../_build/html/_static/panels-variables.06eb56fa6e07937060861dad626602ad.css\n",
      "../_build/html/_static/mystnb.css\n",
      "../_build/html/_static/minus.png\n",
      "../_build/html/_static/copybutton_funcs.js\n",
      "../_build/html/_static/copy-button.svg\n",
      "../_build/html/_static/language_data.js\n",
      "../_build/html/_static/__pycache__\n",
      "../_build/html/_static/__pycache__/__init__.cpython-310.pyc\n",
      "../_build/html/_static/__init__.py\n",
      "../_build/html/_static/jquery-3.5.1.js\n",
      "../_build/html/_static/panda.png\n",
      "../_build/html/_static/copybutton.js\n",
      "../_build/html/_static/file.png\n",
      "../_build/html/_static/clipboard.min.js\n",
      "../_build/html/_static/js\n",
      "../_build/html/_static/js/index.be7d3bbb2ef33a8344ce.js\n",
      "../_build/html/_static/images\n",
      "../_build/html/_static/images/logo_jupyterhub.svg\n",
      "../_build/html/_static/images/logo_binder.svg\n",
      "../_build/html/_static/images/logo_colab.png\n",
      "../_build/html/_static/check-solid.svg\n",
      "../_build/html/_static/css\n",
      "../_build/html/_static/css/index.ff1ffe594081f20da1ef19478df9384b.css\n",
      "../_build/html/_static/css/theme.css\n",
      "../_build/html/_static/css/blank.css\n",
      "../_build/html/_static/searchtools.js\n",
      "../_build/html/_static/documentation_options.js\n",
      "../_build/html/_static/.DS_Store\n",
      "../_build/html/_static/webpack-macros.html\n",
      "../_build/html/_static/underscore.js\n",
      "../_build/html/_static/plus.png\n",
      "../_build/html/algs\n",
      "../_build/html/algs/for-primes-time.html\n",
      "../_build/html/algs/about-primes.html\n",
      "../_build/html/algs/execution-time.html\n",
      "../_build/html/algs/math-and-experiments.html\n",
      "../_build/html/algs/recursion.html\n",
      "../_build/html/algs/loops.html\n",
      "../_build/html/algs/algs-lecture.html\n",
      "../_build/html/algs/primes.html\n",
      "../_build/html/algs/intro.html\n",
      "../_build/html/algs/et.html\n",
      "../_build/html/algs/multiplication.html\n",
      "../_build/html/algs/theory-exercises-1.html\n",
      "../_build/html/algs/programming-exercises-1.html\n",
      "../_build/html/algs/mul.html\n",
      "../_build/html/_sources\n",
      "../_build/html/_sources/overview\n",
      "../_build/html/_sources/overview/overview.md\n",
      "../_build/html/_sources/algs\n",
      "../_build/html/_sources/algs/for-primes-time.md\n",
      "../_build/html/_sources/algs/primes.ipynb\n",
      "../_build/html/_sources/algs/algs-lecture.md\n",
      "../_build/html/_sources/algs/primes.md\n",
      "../_build/html/_sources/algs/recursion.ipynb\n",
      "../_build/html/_sources/algs/loops.ipynb\n",
      "../_build/html/_sources/algs/about-primes.md\n",
      "../_build/html/_sources/algs/mul.ipynb\n",
      "../_build/html/_sources/algs/intro.md\n",
      "../_build/html/_sources/algs/et.ipynb\n",
      "../_build/html/_sources/algs/math-and-experiments.ipynb\n",
      "../_build/html/_sources/algs/theory-exercises-1.md\n",
      "../_build/html/_sources/algs/execution-time.md\n",
      "../_build/html/_sources/algs/programming-exercises-1.ipynb\n",
      "../_build/html/_sources/algs/multiplication.md\n",
      "../_build/html/_sources/week2-divide-and-conquer-and-analysis.ipynb\n",
      "../_build/html/_sources/intro.md\n",
      "../_build/html/_sources/ds\n",
      "../_build/html/_sources/ds/classes.ipynb\n",
      "../_build/html/_sources/ds/queues.ipynb\n",
      "../_build/html/_sources/ds/lecture5.md\n",
      "../_build/html/_sources/ds/lists.ipynb\n",
      "../_build/html/_sources/ds/intro.md\n",
      "../_build/html/_sources/.DS_Store\n",
      "../_build/html/_sources/design\n",
      "../_build/html/_sources/design/dp-intro.md\n",
      "../_build/html/_sources/design/dc-intro.md\n",
      "../_build/html/_sources/design/intro.md\n",
      "../_build/html/_sources/design/dp-fib.ipynb\n",
      "../_build/html/_sources/design/dp-wis.ipynb\n",
      "../_build/html/_sources/design/dp-knapsack.ipynb\n",
      "../_build/html/_sources/design/dc-inversions.ipynb\n",
      "../_build/html/_sources/design/dc-mergesort.ipynb\n",
      "../_build/html/_panels_static\n",
      "../_build/html/_panels_static/panels-main.c949a650a448cc0ae9fd3441c0e17fb0.css\n",
      "../_build/html/_panels_static/panels-variables.06eb56fa6e07937060861dad626602ad.css\n",
      "../_build/html/searchindex.js\n",
      "../_build/html/ds\n",
      "../_build/html/ds/classes.html\n",
      "../_build/html/ds/lecture5.html\n",
      "../_build/html/ds/intro.html\n",
      "../_build/html/ds/lists.html\n",
      "../_build/html/ds/queues.html\n",
      "../_build/html/.DS_Store\n",
      "../_build/html/index.html\n",
      "../_build/html/week2-divide-and-conquer-and-analysis.html\n",
      "../_build/html/design\n",
      "../_build/html/design/dc-mergesort.html\n",
      "../_build/html/design/dp-fib.html\n",
      "../_build/html/design/dp-wis.html\n",
      "../_build/html/design/intro.html\n",
      "../_build/html/design/dc-intro.html\n",
      "../_build/html/design/dp-intro.html\n",
      "../_build/html/design/dc-inversions.html\n",
      "../_build/html/design/dp-knapsack.html\n",
      "../_build/.DS_Store\n",
      "../_toc.yml\n",
      "../algs\n",
      "../algs/primes.ipynb\n",
      "../algs/algs-lecture.md\n",
      "../algs/.ipynb_checkpoints\n",
      "../algs/.ipynb_checkpoints/primes-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/programming-exercises-1-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/mul-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/et-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/loops-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/math-and-experiments-checkpoint.ipynb\n",
      "../algs/.ipynb_checkpoints/recursion-checkpoint.ipynb\n",
      "../algs/recursion.ipynb\n",
      "../algs/loops.ipynb\n",
      "../algs/mul.ipynb\n",
      "../algs/intro.md\n",
      "../algs/et.ipynb\n",
      "../algs/math-and-experiments.ipynb\n",
      "../algs/theory-exercises-1.md\n",
      "../algs/execution-time.md\n",
      "../algs/programming-exercises-1.ipynb\n",
      "../panda.png\n",
      "../intro.md\n",
      "../ds\n",
      "../ds/.ipynb_checkpoints\n",
      "../ds/.ipynb_checkpoints/classes-checkpoint.ipynb\n",
      "../ds/.ipynb_checkpoints/Untitled-checkpoint.ipynb\n",
      "../ds/.ipynb_checkpoints/queues-checkpoint.ipynb\n",
      "../ds/.ipynb_checkpoints/lists-checkpoint.ipynb\n",
      "../ds/classes.ipynb\n",
      "../ds/queues.ipynb\n",
      "../ds/lecture5.md\n",
      "../ds/lists.ipynb\n",
      "../ds/intro.md\n",
      "../design\n",
      "../design/dp-intro.md\n",
      "../design/.ipynb_checkpoints\n",
      "../design/.ipynb_checkpoints/dc-inversions-checkpoint.ipynb\n",
      "../design/.ipynb_checkpoints/dp-fib-checkpoint.ipynb\n",
      "../design/.ipynb_checkpoints/dp-wis-checkpoint.ipynb\n",
      "../design/.ipynb_checkpoints/dp-knapsack-checkpoint.ipynb\n",
      "../design/.ipynb_checkpoints/dc-mergesort-checkpoint.ipynb\n",
      "../design/dc-intro.md\n",
      "../design/intro.md\n",
      "../design/dp-fib.ipynb\n",
      "../design/dp-wis.ipynb\n",
      "../design/dp-knapsack.ipynb\n",
      "../design/dc-inversions.ipynb\n",
      "../design/dc-mergesort.ipynb\n"
     ]
    }
   ],
   "source": [
    "list_dir_tree_stack('..')"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "As you can see it goes as deep as possible (instead of level by level) this algorithm is called DFS: Depth First Search.\n",
    "\n",
    "Make sure you can follow the stack in the example with letters we suggested before. Here is the output you should expect:\n",
    "\n",
    "```text\n",
    "A\n",
    "A/B\n",
    "A/B/e\n",
    "A/B/f\n",
    "A/c\n",
    "A/D\n",
    "A/D/g\n",
    "A/D/H\n",
    "A/D/H/i\n",
    "A/D/H/j\n",
    "```"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.0"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
